@@ -1,4 +1,4 @@
-# Copyrights 2006-2013 by [Mark Overmeer].
+# Copyrights 2006-2014 by [Mark Overmeer].
 #  For other contributors see ChangeLog.
 # See the manual pages for details on the licensing terms.
 # Pod stripped from pm file by OODoc 2.01.
@@ -8,7 +8,7 @@ use strict;
 
 package XML::Compile::Schema::Instance;
 use vars '$VERSION';
-$VERSION = '1.40';
+$VERSION = '1.41';
 
 
 use Log::Report 'xml-compile', syntax => 'SHORT';
@@ -72,7 +72,7 @@ sub complexTypes()    { keys %{shift->{complexType}} }
 sub types()           { ($_[0]->simpleTypes, $_[0]->complexTypes) }
 
 
-my %skip_toplevel = map { ($_ => 1) } qw/annotation notation redefine/;
+my %skip_toplevel = map +($_ => 1), qw/annotation notation redefine/;
 
 sub _collectTypes($$)
 {   my ($self, $schema, $args) = @_;
@@ -103,7 +103,6 @@ sub _collectTypes($$)
 
     $self->{tnses} = {}; # added when used
     $self->{types} = {};
-
     $self->{schema} = $schema;
 
   NODE:
