@@ -1,4 +1,4 @@
-# Copyrights 2006-2013 by [Mark Overmeer].
+# Copyrights 2006-2014 by [Mark Overmeer].
 #  For other contributors see ChangeLog.
 # See the manual pages for details on the licensing terms.
 # Pod stripped from pm file by OODoc 2.01.
@@ -8,7 +8,7 @@ use strict;
 
 package XML::Compile::Schema::NameSpaces;
 use vars '$VERSION';
-$VERSION = '1.40';
+$VERSION = '1.41';
 
 
 use Log::Report 'xml-compile', syntax => 'SHORT';
@@ -45,9 +45,18 @@ sub namespace($)
 sub add(@)
 {   my $self = shift;
     foreach my $instance (@_)
-    {   unshift @{$self->{tns}{$_}}, $instance
-	    for $instance->tnses;
+    {   # With the "new" targetNamespace attribute on any attribute, one
+        # schema may have contribute to multiple tns's.  Also, I have
+        # encounted schema's without elements, but <import>
+        my @tnses = $instance->tnses;
+        @tnses or @tnses = '(none)';
 
+        # newest definitions overrule earlier.
+        unshift @{$self->{tns}{$_}}, $instance
+            for @tnses;
+
+        # inventory where to find definitions which belong to some
+        # substitutionGroup.
         while(my($base,$ext) = each %{$instance->sgs})
         {   $self->{sgs}{$base}{$_} ||= $instance for @$ext;
         }
@@ -242,4 +251,19 @@ sub printIndex(@)
     $self;
 }
 
+
+sub importIndex(%)
+{   my ($self, %args) = @_;
+    my %import;
+    foreach my $fragment (map $self->schemas($_), $self->list)
+    {   foreach my $import ($fragment->imports)
+        {   $import{$import}{$_}++ for $fragment->importLocations($import);
+        }
+    }
+    foreach my $ns (keys %import)
+    {   $import{$ns} = [ grep length, keys %{$import{$ns}} ];
+    }
+    \%import;
+}
+
 1;
