@@ -1,4 +1,4 @@
-# Copyrights 2006-2013 by [Mark Overmeer].
+# Copyrights 2006-2014 by [Mark Overmeer].
 #  For other contributors see ChangeLog.
 # See the manual pages for details on the licensing terms.
 # Pod stripped from pm file by OODoc 2.01.
@@ -8,7 +8,7 @@ no warnings 'recursion';  # trees can be quite deep
 
 package XML::Compile::Translate;
 use vars '$VERSION';
-$VERSION = '1.40';
+$VERSION = '1.41';
 
 
 # Errors are either in _class 'usage': called with request
@@ -220,7 +220,7 @@ sub topLevel($$)
             $node->removeAttribute('form');   # when in schema
             $node->setAttribute(form => 'qualified');
             $elems_qual = 0;
-            $remove_form_attribute = 1;
+            $remove_form_attribute++;
         }
     }
     else {$elems_qual = $qual}
@@ -262,7 +262,7 @@ sub topLevel($$)
     }
 
     $node->removeAttribute('form')
-        if $remove_form_attribute;
+        if $remove_form_attribute && --$remove_form_attribute == 0;
 
     $data;
 }
@@ -1011,12 +1011,12 @@ sub attributeOne($)
     # content: annotation?, simpleType?
 
     my $node = $tree->node;
-    my ($type, $tns);
+    my ($type, $tns, $attr_q);
     local @$self{ qw/elems_qual attrs_qual tns/ }
         = @$self{ qw/elems_qual attrs_qual tns/ };
 
     my($ref, $name, $form, $typeattr);
-    if(my $refattr =  $node->getAttribute('ref'))
+    if(my $refattr = $node->getAttribute('ref'))
     {   my $where  = $tree->path;
         my $refname = $self->rel2abs($tree, $node, $refattr);
         return () if $self->blocked($where, ref => $refname);
@@ -1026,7 +1026,7 @@ sub attributeOne($)
                  , name => $refname, where => $where, _class => 'schema';
 
         $ref        = $def->{node};
-        @$self{ qw/elems_qual attrs_qual tns/ }
+        local @$self{ qw/elems_qual attrs_qual tns/ }
             = $self->nsContext($def);
 
         $name       = $ref->getAttribute('name')
@@ -1043,8 +1043,9 @@ sub attributeOne($)
                      , type => $refname, _class => 'schema';
             $type   = $self->simpleType($other->descend);
         }
-        $form = $ref->getAttribute('form');
-        $node = $ref;
+        $form   = $ref->getAttribute('form');
+        $attr_q = $self->{attrs_qual};
+        $tns    = $self->{tns};
     }
     elsif($tree->nrChildren==1)
     {   $tree->currentLocal eq 'simpleType'
@@ -1052,20 +1053,20 @@ sub attributeOne($)
                  , found => $tree->currentLocal, where => $tree->path
                  , _class => 'schema';
 
-        $name       = $node->getAttribute('name')
+        $name = $node->getAttribute('name')
             or error __x"attribute without name at {where}"
                    , where => $tree->path;
 
-        $form       = $node->getAttribute('form');
-        $type       = $self->simpleType($tree->descend);
+        $form = $node->getAttribute('form');
+        $type = $self->simpleType($tree->descend);
     }
     else
-    {   $name       = $node->getAttribute('name')
+    {   $name = $node->getAttribute('name')
             or error __x"attribute without name or ref at {where}"
                    , where => $tree->path, _class => 'schema';
 
-        $typeattr   = $node->getAttribute('type');
-        $form       = $node->getAttribute('form');
+        $typeattr = $node->getAttribute('type');
+        $form = $node->getAttribute('form');
     }
 
     my $where = $tree->path.'/@'.$name;
@@ -1085,10 +1086,10 @@ sub attributeOne($)
              , where => $where, _class => 'schema';
 
     my $qual
-      = ! defined $form        ? $self->{attrs_qual}
+      = ! defined $form        ? ($attr_q || $self->{attrs_qual})
       : $form eq 'qualified'   ? 1
       : $form eq 'unqualified' ? 0
-      : error __x"form must be (un)qualified, not {form} at {where}"
+      : error __x"form must be (un)qualified, not `{form}' at {where}"
             , form => $form, where => $where, _class => 'schema';
 
     $tns      ||= $node->getAttribute('targetNamespace') || $self->{tns};
